__author__ = "Nitin Kumar, Rick Sherman"
__credits__ = "Jeremy Schulman"

import unittest
from unittest.mock import patch
import nose2

from jnpr.junos import Device
from jnpr.junos.ofacts.switch_style import facts_switch_style as switch_style


class TestSwitchStyle(unittest.TestCase):
    @patch("jnpr.junos.device.warnings")
    def setUp(self, mock_warnings):
        self.facts = {}
        self.dev = Device(
            host="1.1.1.1",
            user="rick",
            password="password123",
            gather_facts=False,
            fact_style="old",
        )

    def test_switch_style_mx_srx(self):
        self.facts["personality"] = "SRX_HIGHEND"
        switch_style(self.dev, self.facts)
        self.assertEqual(self.facts["switch_style"], "BRIDGE_DOMAIN")

    def test_switch_style_model_firefly(self):
        self.facts["personality"] = "SWITCH"
        self.facts["model"] = "FIReFly"
        switch_style(self.dev, self.facts)
        self.assertEqual(self.facts["switch_style"], "NONE")

    def test_switch_style_model_ex9000(self):
        self.facts["personality"] = "SWITCH"
        self.facts["model"] = "EX9000"
        switch_style(self.dev, self.facts)
        self.assertEqual(self.facts["switch_style"], "VLAN_L2NG")

    def test_switch_style_model_not_specific(self):
        self.facts["personality"] = "SWITCH"
        self.facts["model"] = "abc"
        switch_style(self.dev, self.facts)
        self.assertEqual(self.facts["switch_style"], "VLAN")

    def test_switch_style_persona_not_specific(self):
        self.facts["personality"] = "PTX"
        self.facts["model"] = "abc"
        switch_style(self.dev, self.facts)
        self.assertEqual(self.facts["switch_style"], "NONE")



__author__ = "Stacy Smith"
__credits__ = "Jeremy Schulman, Nitin Kumar"

import unittest
import nose2
from unittest.mock import patch, MagicMock
import os
from lxml import etree

from jnpr.junos import Device
from jnpr.junos.exception import RpcError

from ncclient.manager import Manager, make_device_handler
from ncclient.transport import SSHSession


class TestGetSoftwareInformation(unittest.TestCase):
    @patch("ncclient.manager.connect")
    def setUp(self, mock_connect):
        mock_connect.side_effect = self._mock_manager_setup
        self.dev = Device(
            host="1.1.1.1", user="rick", password="password123", gather_facts=False
        )
        self.dev.open()

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_single(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_single
        self.assertEqual(self.dev.facts["junos_info"]["re0"]["text"], "16.1R1.11")
        self.assertEqual(
            self.dev.facts["junos_info"]["re0"]["object"].as_tuple,
            (16, 1, "R", "1", 11),
        )
        self.assertEqual(self.dev.facts["hostname"], "r0")
        self.assertEqual(self.dev.facts["model"], "MX960")
        self.assertEqual(self.dev.facts["model_info"], {"re0": "MX960"})
        self.assertEqual(self.dev.facts["version"], "16.1R1.11")
        self.assertEqual(self.dev.facts["version_info"].as_tuple, (16, 1, "R", "1", 11))
        self.assertEqual(self.dev.facts["version_RE0"], "16.1R1.11")
        self.assertEqual(self.dev.facts["version_RE1"], None)

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_vc(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_vc
        self.assertEqual(
            self.dev.facts["junos_info"]["member0"]["text"], "15.1-20161209.0"
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["member0"]["object"].as_tuple,
            (15, 1, "I", "20161209", "0"),
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["member1"]["text"], "15.1-20161209.0"
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["member1"]["object"].as_tuple,
            (15, 1, "I", "20161209", "0"),
        )
        self.assertEqual(self.dev.facts["hostname"], "reefbreak1")
        self.assertEqual(self.dev.facts["model"], "MX240")
        self.assertEqual(
            self.dev.facts["model_info"], {"member1": "MX240", "member0": "MX240"}
        )
        self.assertEqual(self.dev.facts["version"], "15.1-20161209.0")
        self.assertEqual(
            self.dev.facts["version_info"].as_tuple, (15, 1, "I", "20161209", "0")
        )
        self.assertEqual(self.dev.facts["version_RE0"], None)
        self.assertEqual(self.dev.facts["version_RE1"], None)

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_simple(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_simple
        self.assertEqual(self.dev.facts["junos_info"]["re0"]["text"], "12.3X48-D40.5")
        self.assertEqual(
            self.dev.facts["junos_info"]["re0"]["object"].as_tuple,
            (12, 3, "X", (48, "D", 40), 5),
        )
        self.assertEqual(self.dev.facts["hostname"], "lsys500")
        self.assertEqual(self.dev.facts["model"], "SRX3600")
        self.assertEqual(self.dev.facts["model_info"], {"re0": "SRX3600"})
        self.assertEqual(self.dev.facts["version"], "12.3X48-D40.5")
        self.assertEqual(
            self.dev.facts["version_info"].as_tuple, (12, 3, "X", (48, "D", 40), 5)
        )
        self.assertEqual(self.dev.facts["version_RE0"], "12.3X48-D40.5")
        self.assertEqual(self.dev.facts["version_RE1"], None)

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_no_version(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_no_version
        self.assertEqual(self.dev.facts["junos_info"], None)
        self.assertEqual(self.dev.facts["hostname"], "lsys500")
        self.assertEqual(self.dev.facts["model"], "SRX3600")
        self.assertEqual(self.dev.facts["model_info"], {"re0": "SRX3600"})
        self.assertEqual(self.dev.facts["version"], "0.0I0.0")
        self.assertEqual(self.dev.facts["version_RE0"], None)
        self.assertEqual(self.dev.facts["version_RE1"], None)

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_dual(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_dual
        self.assertEqual(self.dev.facts["junos_info"]["re0"]["text"], "15.1F5.15-C1.12")
        self.assertEqual(self.dev.facts["junos_info"]["re1"]["text"], "15.1F5.15")
        self.assertEqual(self.dev.facts["hostname"], "baku")
        self.assertEqual(self.dev.facts["model"], "MX480")
        self.assertEqual(self.dev.facts["model_info"], {"re0": "MX480", "re1": "MX480"})
        self.assertEqual(self.dev.facts["version"], "15.1F5.15-C1.12")
        self.assertEqual(self.dev.facts["version_RE0"], "15.1F5.15-C1.12")
        self.assertEqual(self.dev.facts["version_RE1"], "15.1F5.15")

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_dual_other_re_off(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_dual_other_re_off
        self.assertEqual(
            self.dev.facts["junos_info"]["re1"]["text"], "18.3I20180716_1639"
        )
        self.assertEqual(self.dev.facts["hostname"], "R1_re01")
        self.assertEqual(self.dev.facts["model"], "MX960")
        self.assertEqual(self.dev.facts["model_info"], {"re1": "MX960"})
        self.assertEqual(self.dev.facts["version"], "18.3I20180716_1639")
        self.assertEqual(self.dev.facts["version_RE1"], "18.3I20180716_1639")

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_txp(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_txp
        self.assertEqual(self.dev.facts["junos_info"]["re0"]["text"], "13.3R9-S2.1")
        self.assertEqual(self.dev.facts["hostname"], "dj")
        self.assertEqual(self.dev.facts["model"], "T1600")
        self.assertEqual(self.dev.facts["model_info"], {"re0": "T1600"})
        self.assertEqual(self.dev.facts["version"], "13.3R9-S2.1")
        self.assertEqual(self.dev.facts["version_RE0"], "13.3R9-S2.1")
        self.assertEqual(self.dev.facts["version_RE1"], None)

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_ex(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_ex
        self.assertEqual(self.dev.facts["junos_info"]["re0"]["text"], "11.4R1.6")
        self.assertEqual(self.dev.facts["hostname"], "sw1")
        self.assertEqual(self.dev.facts["model"], "EX2200-C-12T-2G")
        self.assertEqual(self.dev.facts["model_info"], {"re0": "EX2200-C-12T-2G"})
        self.assertEqual(self.dev.facts["version"], "11.4R1.6")
        self.assertEqual(self.dev.facts["version_RE0"], "11.4R1.6")
        self.assertEqual(self.dev.facts["version_RE1"], None)

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_nfx(self, mock_execute):
        self.dev.facts._cache["vc_capable"] = False
        mock_execute.side_effect = self._mock_manager_nfx
        self.assertEqual(self.dev.facts["hostname"], "jdm")
        self.assertEqual(self.dev.facts["model"], "NFX250_S2_10_T")
        self.assertEqual(self.dev.facts["version"], "15.1X53-D45.3")
        self.assertEqual(self.dev.facts["version_RE0"], "15.1X53-D45.3")
        self.assertEqual(self.dev.facts["version_RE1"], None)
        self.assertEqual(self.dev.facts["model_info"], {"re0": "NFX250_S2_10_T"})
        self.assertEqual(self.dev.facts["junos_info"]["re0"]["text"], "15.1X53-D45.3")

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_bsys(self, mock_execute):
        self.dev.facts._cache["current_re"] = [
            "master",
            "node",
            "fwdd",
            "member",
            "pfem",
            "re0",
        ]
        self.dev.facts._cache["vc_capable"] = False
        mock_execute.side_effect = self._mock_manager_bsys
        self.assertEqual(self.dev.facts["hostname"], "bsys")
        self.assertEqual(self.dev.facts["model"], "MX2020")
        self.assertEqual(self.dev.facts["version"], "17.4-20170706_dev_common.0")
        self.assertEqual(self.dev.facts["version_RE0"], "17.4-20170706_dev_common.0")
        self.assertEqual(self.dev.facts["version_RE1"], "17.4-20170706_dev_common.0")
        self.assertEqual(
            self.dev.facts["model_info"],
            {
                "bsys-re0": "MX2020",
                "bsys-re1": "MX2020",
                "gnf1-re0": "MX2020",
                "gnf1-re1": "MX2020",
                "gnf2-re0": "MX2020",
                "gnf2-re1": "MX2020",
                "gnf3-re0": "MX2020",
                "gnf3-re1": "MX2020",
                "gnf4-re0": "MX2020",
                "gnf4-re1": "MX2020",
            },
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["bsys-re0"]["text"],
            "17.4-20170706_dev_common.0",
        )

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_jdm(self, mock_execute):
        self.dev.facts._cache["vc_capable"] = False
        self.dev.facts._cache["current_re"] = ["server0"]
        mock_execute.side_effect = self._mock_manager_jdm
        self.assertEqual(self.dev.facts["hostname"], "jdm")
        self.assertEqual(self.dev.facts["model"], "JUNOS_NODE_SLICING")
        self.assertEqual(self.dev.facts["version"], "17.4-20170718_dev_common.1-secure")
        self.assertEqual(
            self.dev.facts["version_RE0"], "17.4-20170718_dev_common.1-secure"
        )
        self.assertEqual(
            self.dev.facts["version_RE1"], "17.4-20170718_dev_common.1-secure"
        )
        self.assertEqual(
            self.dev.facts["model_info"],
            {"server0": "JUNOS_NODE_SLICING", "server1": "JUNOS_NODE_SLICING"},
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["server0"]["text"],
            "17.4-20170718_dev_common.1-secure",
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["server1"]["text"],
            "17.4-20170718_dev_common.1-secure",
        )

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_gnf(self, mock_execute):
        self.dev.facts._cache["vc_capable"] = False
        self.dev.facts._cache["current_re"] = ["gnf7-master", "gnf7-re0"]
        mock_execute.side_effect = self._mock_manager_gnf
        self.assertEqual(self.dev.facts["hostname"], "mgb-gnf-a")
        self.assertEqual(self.dev.facts["model"], "MX960")
        self.assertEqual(self.dev.facts["version"], "18.4-20180707_dev_common.0")
        self.assertEqual(self.dev.facts["version_RE0"], "18.4-20180707_dev_common.0")
        self.assertEqual(self.dev.facts["version_RE1"], "18.4-20180707_dev_common.0")
        self.assertEqual(
            self.dev.facts["model_info"],
            {
                "bsys-re0": "MX960",
                "bsys-re1": "MX960",
                "gnf7-re0": "MX960",
                "gnf7-re1": "MX960",
            },
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["gnf7-re0"]["text"],
            "18.4-20180707_dev_common.0",
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["gnf7-re1"]["text"],
            "18.4-20180707_dev_common.0",
        )

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_srx_cluster(self, mock_execute):
        self.dev.facts._cache["vc_capable"] = False
        self.dev.facts._cache["current_re"] = ["node0"]
        mock_execute.side_effect = self._mock_manager_srx_cluster
        self.assertEqual(self.dev.facts["hostname"], "frogbert")
        self.assertEqual(self.dev.facts["model"], "SRX5800")
        self.assertEqual(
            self.dev.facts["version"], "17.3-2017-07-02.0_RELEASE_173_THROTTLE"
        )
        self.assertEqual(
            self.dev.facts["version_RE0"], "17.3-2017-07-02.0_RELEASE_173_THROTTLE"
        )
        self.assertEqual(
            self.dev.facts["version_RE1"], "17.3-2017-07-02.0_RELEASE_173_THROTTLE"
        )
        self.assertEqual(
            self.dev.facts["model_info"], {"node0": "SRX5800", "node1": "SRX5800"}
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["node0"]["text"],
            "17.3-2017-07-02.0_RELEASE_173_THROTTLE",
        )
        self.assertEqual(
            self.dev.facts["junos_info"]["node1"]["text"],
            "17.3-2017-07-02.0_RELEASE_173_THROTTLE",
        )

    @patch("jnpr.junos.Device.execute")
    def test_sw_info_err(self, mock_execute):
        self.dev.facts._cache["vc_capable"] = False
        self.dev.facts._cache["current_re"] = None
        mock_execute.side_effect = self._mock_manager_err
        self.assertEqual(self.dev.facts["hostname"], None)
        self.assertEqual(self.dev.facts["model"], None)
        self.assertEqual(self.dev.facts["version"], None)
        self.assertEqual(self.dev.facts["version_RE0"], None)
        self.assertEqual(self.dev.facts["version_RE1"], None)
        self.assertEqual(self.dev.facts["model_info"], None)
        self.assertEqual(self.dev.facts["junos_info"], None)

    def _read_file(self, fname):
        from ncclient.xml_ import NCElement

        fpath = os.path.join(os.path.dirname(__file__), "rpc-reply", fname)
        foo = open(fpath).read()

        rpc_reply = NCElement(
            foo, self.dev._conn._device_handler.transform_reply()
        )._NCElement__doc[0]
        return rpc_reply

    def _mock_manager_setup(self, *args, **kwargs):
        if kwargs:
            device_params = kwargs["device_params"]
            device_handler = make_device_handler(device_params)
            session = SSHSession(device_handler)
            return Manager(session, device_handler)

    def _mock_manager_single(self, *args, **kwargs):
        if args:
            return self._read_file("sw_info_single_" + args[0].tag + ".xml")

    def _mock_manager_vc(self, *args, **kwargs):
        if args:
            if (
                args[0].tag == "command"
                and args[0].text == "show version invoke-on all-routing-engines"
            ):
                raise RpcError()
            elif (
                args[0].tag == "file-show"
                and args[0].xpath("filename")[0].text == "/usr/share/cevo/cevo_version"
            ):
                raise RpcError
            else:
                return self._read_file("sw_info_vc_" + args[0].tag + ".xml")

    def _mock_manager_simple(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            else:
                return self._read_file("sw_info_simple_" + args[0].tag + ".xml")

    def _mock_manager_no_version(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            else:
                return self._read_file("sw_info_no_version_" + args[0].tag + ".xml")

    def _mock_manager_dual(self, *args, **kwargs):
        if args:
            if (
                args[0].tag == "file-show"
                and args[0].xpath("filename")[0].text == "/usr/share/cevo/cevo_version"
            ):
                raise RpcError
            else:
                return self._read_file("sw_info_dual_" + args[0].tag + ".xml")

    def _mock_manager_dual_other_re_off(self, *args, **kwargs):
        if args:
            return self._read_file("sw_info_dual_other_re_off.xml").getparent()

    def _mock_manager_txp(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            else:
                return self._read_file("sw_info_txp_" + args[0].tag + ".xml")

    def _mock_manager_ex(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            else:
                return self._read_file("sw_info_ex_" + args[0].tag + ".xml")

    def _mock_manager_nfx(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            elif (
                args[0].tag == "get-software-information"
                and args[0].find("./*") is None
            ):
                return True
            else:
                return self._read_file("sw_info_nfx_" + args[0].tag + ".xml")

    def _mock_manager_bsys(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            elif (
                args[0].tag == "get-software-information"
                and args[0].find("./*") is None
            ):
                return True
            else:
                return self._read_file("sw_info_bsys_" + args[0].tag + ".xml")

    def _mock_manager_srx_cluster(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            else:
                return self._read_file("sw_info_srx_cluster_" + args[0].tag + ".xml")

    def _mock_manager_jdm(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                if args[0].text == "show version invoke-on all-routing-engines":
                    raise RpcError()
                else:
                    return self._read_file(
                        "sw_info_jdm_command_" + args[0].text + ".xml"
                    )
            else:
                return self._read_file("sw_info_jdm_" + args[0].tag + ".xml")

    def _mock_manager_gnf(self, *args, **kwargs):
        if args:
            if args[0].tag == "command":
                raise RpcError()
            elif (
                args[0].tag == "get-software-information"
                and args[0].find("./*") is None
            ):
                return True
            else:
                return self._read_file("sw_info_gnf_" + args[0].tag + ".xml")

    def _mock_manager_err(self, *args, **kwargs):
        if args:
            raise RpcError()



__author__ = "Stacy Smith"
__credits__ = "Jeremy Schulman, Nitin Kumar"

import unittest
import nose2
from unittest.mock import patch, MagicMock
import os
from lxml import etree

from jnpr.junos import Device

from ncclient.manager import Manager, make_device_handler
from ncclient.transport import SSHSession


class TestGetRouteEngineInformation(unittest.TestCase):
    @patch("ncclient.manager.connect")
    def setUp(self, mock_connect):
        mock_connect.side_effect = self._mock_manager_setup
        self.dev = Device(
            host="1.1.1.1", user="rick", password="password123", gather_facts=False
        )
        self.dev.open()

    @patch("jnpr.junos.Device.execute")
    def test_re_info_dual(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_dual_re
        self.assertEqual(self.dev.facts["2RE"], True)
        self.assertEqual(self.dev.facts["master"], "RE0")
        self.assertEqual(
            self.dev.facts["RE0"],
            {
                "status": "OK",
                "last_reboot_reason": "Router rebooted after a normal shutdown.",
                "model": "RE-S-1800x4",
                "up_time": "9 days, 22 hours, 27 minutes, 12 seconds",
                "mastership_state": "master",
            },
        )
        self.assertEqual(
            self.dev.facts["RE1"],
            {
                "status": "OK",
                "last_reboot_reason": "Router rebooted after a normal shutdown.",
                "model": "RE-S-1800x4",
                "up_time": "9 days, 22 hours, 26 minutes, 48 seconds",
                "mastership_state": "backup",
            },
        )
        self.assertEqual(
            self.dev.facts["re_info"],
            {
                "default": {
                    "1": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "backup",
                    },
                    "0": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "master",
                    },
                    "default": {
                        "status": "OK",
                        "last_reboot_reason": "Router rebooted after a "
                        "normal shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "master",
                    },
                }
            },
        )
        self.assertEqual(self.dev.facts["re_master"], {"default": "0"})

    @patch("jnpr.junos.Device.execute")
    def test_re_info_single(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_single_re
        self.assertEqual(self.dev.facts["2RE"], False)
        self.assertEqual(self.dev.facts["master"], "RE0")
        self.assertEqual(
            self.dev.facts["RE0"],
            {
                "status": "OK",
                "last_reboot_reason": "Router rebooted after a normal shutdown.",
                "model": "RE-VMX",
                "up_time": "29 days, 22 hours, 41 minutes, 35 seconds",
                "mastership_state": "master",
            },
        )
        self.assertEqual(self.dev.facts["RE1"], None)
        self.assertEqual(
            self.dev.facts["re_info"],
            {
                "default": {
                    "default": {
                        "status": "OK",
                        "last_reboot_reason": "Router rebooted "
                        "after a normal "
                        "shutdown.",
                        "model": "RE-VMX",
                        "mastership_state": "master",
                    },
                    "0": {
                        "status": "OK",
                        "last_reboot_reason": "Router rebooted after a "
                        "normal shutdown.",
                        "model": "RE-VMX",
                        "mastership_state": "master",
                    },
                }
            },
        )
        self.assertEqual(self.dev.facts["re_master"], {"default": "0"})

    @patch("jnpr.junos.Device.execute")
    def test_re_info_mx_vc(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_mx_vc
        self.assertEqual(self.dev.facts["2RE"], True)
        self.assertEqual(self.dev.facts["master"], ["RE1", "RE0"])
        self.assertEqual(
            self.dev.facts["RE0"],
            {
                "status": "OK",
                "last_reboot_reason": "Router rebooted after a normal shutdown.",
                "model": "RE-S-1800x4",
                "up_time": "16 days, 13 hours, 17 minutes, 25 seconds",
                "mastership_state": "backup",
            },
        )
        self.assertEqual(
            self.dev.facts["RE1"],
            {
                "status": "OK",
                "last_reboot_reason": "Router rebooted after a normal shutdown.",
                "model": "RE-S-1800x4",
                "up_time": "18 days, 22 hours, 3 minutes, 18 seconds",
                "mastership_state": "master",
            },
        )
        self.assertEqual(
            self.dev.facts["re_info"],
            {
                "default": {
                    "1": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "master",
                    },
                    "0": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "backup",
                    },
                    "default": {
                        "status": "OK",
                        "last_reboot_reason": "Router rebooted after "
                        "a normal shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "backup",
                    },
                },
                "member1": {
                    "1": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "backup",
                    },
                    "0": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "master",
                    },
                },
                "member0": {
                    "1": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "master",
                    },
                    "0": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "backup",
                    },
                    "default": {
                        "status": "OK",
                        "last_reboot_reason": "Router "
                        "rebooted "
                        "after a "
                        "normal "
                        "shutdown.",
                        "model": "RE-S-1800x4",
                        "mastership_state": "backup",
                    },
                },
            },
        )

    def _read_file(self, fname):
        from ncclient.xml_ import NCElement

        fpath = os.path.join(os.path.dirname(__file__), "rpc-reply", fname)
        foo = open(fpath).read()

        rpc_reply = NCElement(
            foo, self.dev._conn._device_handler.transform_reply()
        )._NCElement__doc[0]
        return rpc_reply

    def _mock_manager_setup(self, *args, **kwargs):
        if kwargs:
            device_params = kwargs["device_params"]
            device_handler = make_device_handler(device_params)
            session = SSHSession(device_handler)
            return Manager(session, device_handler)

    def _mock_manager_dual_re(self, *args, **kwargs):
        if args:
            return self._read_file("re_info_dual_" + args[0].tag + ".xml")

    def _mock_manager_single_re(self, *args, **kwargs):
        if args:
            return self._read_file("re_info_single_" + args[0].tag + ".xml")

    def _mock_manager_mx_vc(self, *args, **kwargs):
        if args:
            return self._read_file("re_info_mx_vc_" + args[0].tag + ".xml")



__author__ = "Stacy Smith"
__credits__ = "Jeremy Schulman, Nitin Kumar"

try:
    import unittest2 as unittest
except:
    import unittest
import nose2
from unittest.mock import patch, MagicMock
import os

from jnpr.junos import Device

from ncclient.manager import Manager, make_device_handler
from ncclient.transport import SSHSession


class TestVmHost(unittest.TestCase):
    @patch("ncclient.manager.connect")
    def setUp(self, mock_connect):
        mock_connect.side_effect = self._mock_manager_setup
        self.dev = Device(
            host="1.1.1.1", user="rick", password="password123", gather_facts=False
        )
        self.dev.open()

    @patch("jnpr.junos.Device.execute")
    def test_vmhost_true(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_vmhost_true
        self.assertEqual(self.dev.facts["vmhost"], True)

    @patch("jnpr.junos.Device.execute")
    def test_vmhost_false(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_vmhost_false
        self.assertEqual(self.dev.facts["vmhost"], False)

    @patch("jnpr.junos.Device.execute")
    def test_vmhost_linux(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_vmhost_false
        self.dev.facts._cache["_is_linux"] = True
        self.assertEqual(self.dev.facts["vmhost"], False)

    @patch("jnpr.junos.Device.execute")
    def test_vmhost_error(self, mock_execute):
        mock_execute.side_effect = self._mock_manager_vmhost_error
        self.assertEqual(self.dev.facts["vmhost"], None)

    def _read_file(self, fname):
        from ncclient.xml_ import NCElement

        fpath = os.path.join(os.path.dirname(__file__), "rpc-reply", fname)
        foo = open(fpath).read()

        rpc_reply = NCElement(
            foo, self.dev._conn._device_handler.transform_reply()
        )._NCElement__doc[0]
        return rpc_reply

    def _mock_manager_setup(self, *args, **kwargs):
        if kwargs:
            device_params = kwargs["device_params"]
            device_handler = make_device_handler(device_params)
            session = SSHSession(device_handler)
            return Manager(session, device_handler)

    def _mock_manager_vmhost_true(self, *args, **kwargs):
        if args:
            return self._read_file("vmhost_true_" + args[0].tag + ".xml")

    def _mock_manager_vmhost_false(self, *args, **kwargs):
        if args:
            return self._read_file("vmhost_false_" + args[0].tag + ".xml")

    def _mock_manager_vmhost_error(self, *args, **kwargs):
        if args:
            return self._read_file("vmhost_error_" + args[0].tag + ".xml")



__author__ = "Rick Sherman, Nitin Kumar"
__credits__ = "Jeremy Schulman"

import unittest
import nose2
from jnpr.junos.factory import FactoryLoader
from unittest.mock import patch


class TestFactoryLoader(unittest.TestCase):
    def setUp(self):
        self.fl = FactoryLoader()
        self.fl.catalog = {
            "RouteTable": {
                "item": "route-table/rt",
                "rpc": "get-route-information",
                "args_key": "destination",
                "key": "rt-destination",
                "view": "RouteTableView",
            }
        }

        self.fl._catalog_dict = {
            "RouteTableView": {
                "groups": {"entry": "rt-entry"},
                "fields_entry": {
                    "via": "nh/via | nh/nh-local-interface",
                    "age": {"age/@seconds": "int"},
                    "nexthop": "nh/to",
                    "protocol": "protocol-name",
                },
                "extends": "test",
            }
        }

    def test_FactoryLoader__build_optable(self):
        self.assertEqual(
            self.fl._build_optable("RouteTable"), self.fl.catalog["RouteTable"]
        )

    def test_FactoryLoader__build_cfgtable(self):
        self.assertEqual(
            self.fl._build_cfgtable("RouteTable"), self.fl.catalog["RouteTable"]
        )

    @patch("jnpr.junos.factory.factory_loader._VIEW")
    def test_FactoryLoader__build_view(self, mock_view):
        mock_view.return_value = type("test", (object,), {})
        self.assertEqual(
            self.fl._build_view("RouteTableView"), self.fl.catalog["RouteTableView"]
        )

    def test_FactoryLoader__fieldfunc_True(self):
        fn = self.fl._fieldfunc_True("test")
        self.assertTrue(fn("test"))

    def test_FactoryLoader__fieldfunc_True_fail(self):
        # Assert that false is returned if value not found
        fn = self.fl._fieldfunc_True("test")
        self.assertFalse(fn("nope"))

    def test_FactoryLoader__fieldfunc_True_regex(self):
        fn = self.fl._fieldfunc_True("regex(this|test)")
        self.assertTrue(fn("test"))

    def test_FactoryLoader__fieldfunc_True_regex_fail(self):
        # Assert that false is returned if value not found
        fn = self.fl._fieldfunc_True("regex(this|test)")
        self.assertFalse(fn("nope"))

    def test_FactoryLoader__fieldfunc_False(self):
        fn = self.fl._fieldfunc_False("test")
        self.assertFalse(fn("test"))

    def test_FactoryLoader__fieldfunc_False_fail(self):
        # Assert that true is returned if value not found
        fn = self.fl._fieldfunc_False("test")
        self.assertTrue(fn("nope"))

    def test_FactoryLoader__fieldfunc_False_regex(self):
        fn = self.fl._fieldfunc_False("regex(this|test)")
        self.assertFalse(fn("test"))

    def test_FactoryLoader__fieldfunc_False_regex_fail(self):
        # Assert that true is returned if value not found
        fn = self.fl._fieldfunc_False("regex(this|test)")
        self.assertTrue(fn("nope"))

    def test_FactoryLoader__add_dictfield_RuntimeError(self):
        self.assertRaises(
            RuntimeError,
            self.fl._add_dictfield,
            "testing",
            "age",
            {"age/@seconds": "test"},
            {},
        )

    def test_FactoryLoader__add_dictfield_ValueError(self):
        self.assertRaises(
            ValueError,
            self.fl._add_dictfield,
            "testing",
            "age",
            {"age/@seconds": "test=test"},
            {},
        )



__author__ = "Rick Sherman"
__credits__ = "Jeremy Schulman"

import unittest
import os
import yaml
import json
import nose2

from jnpr.junos import Device
from jnpr.junos.op.phyport import PhyPortStatsTable
from jnpr.junos.op.ethport import EthPortTable
from jnpr.junos.factory.factory_loader import FactoryLoader
from jnpr.junos.factory.optable import generate_sax_parser_input

from ncclient.manager import Manager, make_device_handler
from ncclient.transport import SSHSession
from ncclient.operations.rpc import RPCReply

from lxml import etree

from unittest.mock import patch


class TestFactoryOpTable(unittest.TestCase):
    @patch("ncclient.manager.connect")
    def setUp(self, mock_connect):
        mock_connect.side_effect = self._mock_manager
        self.dev = Device(
            host="1.1.1.1", user="rick", password="password123", gather_facts=False
        )
        self.dev.open()
        self.ppt = PhyPortStatsTable(self.dev)

    @patch("jnpr.junos.Device.execute")
    def test_optable_get(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.ppt.get()
        self.assertEqual(len(self.ppt), 2)

    @patch("jnpr.junos.Device.execute")
    def test_optable_get_key(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.ppt.get("ge-0/0/0")
        self.assertEqual(self.ppt.GET_KEY, "interface_name")

    def test_optable_path(self):
        fname = "local-get-interface-information.xml"
        path = os.path.join(os.path.dirname(__file__), "rpc-reply", fname)
        lppt = PhyPortStatsTable(path=path)
        lppt.get()
        self.assertEqual(len(lppt), 2)

    def test_optable_xml(self):
        fname = "get-interface-information.xml"
        xml = self._read_file(fname)
        lppt = PhyPortStatsTable(xml=xml)
        lppt.get()
        self.assertEqual(len(lppt), 2)

    @patch("jnpr.junos.Device.execute")
    def test_optable_view_get(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.ppt.get()
        v = self.ppt["ge-0/0/0"]
        self.assertEqual(v["rx_packets"], 1207)

    @patch("jnpr.junos.Device.execute")
    def test_optable_view_get_astype_bool(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        et = EthPortTable(self.dev)
        et.get()
        v = et["ge-0/0/0"]
        self.assertEqual(v["present"], True)

    @patch("jnpr.junos.Device.execute")
    def test_optable_view_get_astype_bool_regex(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        from jnpr.junos.op.bfd import BfdSessionTable

        bfd = BfdSessionTable(self.dev)
        bfd.get()
        v = bfd["10.92.20.4"]
        self.assertEqual(v["no_absorb"], True)

    @patch("jnpr.junos.Device.execute")
    def test_optable_view_get_unknown_field(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.ppt.get()

        def bad(key):
            v = self.ppt["ge-0/0/0"]
            return v[key]

        self.assertRaises(ValueError, bad, "bunk")

    def test_generate_sax_parser_fields_with_many_slash(self):
        yaml_data = """
---
bgpNeighborTable:
    rpc: get-bgp-neighbor-information
    item: bgp-peer
    key: peer-address
    view: bgpNeighborView
bgpNeighborView:
    fields:
        prefix-count: bgp-option-information/prefix-limit/prefix-count
        prefix-dummy: bgp-option-information/prefix-limit/prefix-dummy
"""
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = bgpNeighborTable(self.dev)
        data = generate_sax_parser_input(tbl)
        self.assertEqual(data.tag, "bgp-peer")
        self.assertEqual(
            len(etree.tostring(data)),
            len(
                b"<bgp-peer><peer-address/><bgp-option-information><prefix-limit>"
                b"<prefix-count/><prefix-dummy/></prefix-limit>"
                b"</bgp-option-information></bgp-peer>"
            ),
        )

    def test_generate_sax_parser_fields_with_diff_child_xpaths(self):
        yaml_data = """
---
twampProbeTable:
    rpc: twamp-get-probe-results
    item: probe-test-results
    key: test-name
    view: probeResultsView
probeResultsView:
    fields:
        min-delay: probe-test-global-results/probe-test-generic-results/probe-test-rtt/probe-summary-results/min-delay
        max-delay: probe-test-global-results/probe-test-generic-results/probe-test-rtt/probe-summary-results/max-delay
        avg-delay: probe-test-global-results/probe-test-generic-results/probe-test-rtt/probe-summary-results/avg-delay
        positive-rtt-jitter: probe-test-global-results/probe-test-generic-results/probe-test-positive-round-trip-jitter/probe-summary-results/avg-delay
        loss-percentage: probe-test-global-results/probe-test-generic-results/loss-percentage
        current-min-delay: probe-last-test-results/probe-test-generic-results/probe-test-rtt/probe-summary-results/min-delay
        current-max-delay: probe-last-test-results/probe-test-generic-results/probe-test-rtt/probe-summary-results/max-delay
        current-avg-delay: probe-last-test-results/probe-test-generic-results/probe-test-rtt/probe-summary-results/avg-delay
        current-positive-rtt-jitter: probe-last-test-results/probe-test-generic-results/probe-test-positive-round-trip-jitter/probe-summary-results/avg-delay
        current-loss-percentage: probe-last-test-results/probe-test-generic-results/loss-percentage
"""
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = twampProbeTable(self.dev)
        data = generate_sax_parser_input(tbl)
        self.assertEqual(data.tag, "probe-test-results")
        self.assertEqual(
            len(etree.tostring(data)),
            len(
                b"<probe-test-results><test-name/><probe-last-test-results>"
                b"<probe-test-generic-results><probe-test-rtt><probe-summary"
                b"-results><min-delay/><avg-delay/><max-delay/></probe-summary"
                b"-results></probe-test-rtt><loss-percentage/>"
                b"<probe-test-positive-round-trip-jitter><probe-summary-results>"
                b"<avg-delay/></probe-summary-results></probe-test-positive-round"
                b"-trip-jitter></probe-test-generic-results></probe-last-test-resu"
                b"lts><probe-test-global-results><probe-test-generic-results>"
                b"<probe-test-rtt><probe-summary-results><min-delay/><avg-delay/>"
                b"<max-delay/></probe-summary-results></probe-test-rtt>"
                b"<loss-percentage/><probe-test-positive-round-trip-jitter>"
                b"<probe-summary-results><avg-delay/></probe-summary-results>"
                b"</probe-test-positive-round-trip-jitter></probe-test-generic-"
                b"results></probe-test-global-results></probe-test-results>"
            ),
        )

    def test_generate_sax_parser_item_with_many_slash(self):
        yaml_data = """
---
taskmallocdetail:
    rpc: get-task-memory-information
    args:
        level: detail
    item: task-memory-malloc-usage-report/task-malloc-list/task-malloc
    key: tm-name
    view: taskmallocview

taskmallocview:
    fields:
        tmallocs: tm-allocs
        tmallocbytes: tm-alloc-bytes
        tmmaxallocs: tm-max-allocs
        tmmaxallocbytes: tm-max-alloc-bytes
        tmfunctioncalls: tm-function-calls
"""
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = taskmallocdetail(self.dev)
        data = generate_sax_parser_input(tbl)
        self.assertEqual(data.tag, "task-memory-malloc-usage-report")
        self.assertEqual(
            len(etree.tostring(data)),
            len(
                b"<task-memory-malloc-usage-report><task-malloc-list><task-malloc><tm-name/><t"
                b"m-allocs/><tm-alloc-bytes/><tm-max-allocs/><tm-max-alloc-bytes/><tm-function"
                b"-calls/></task-malloc></task-malloc-list></task-memory-malloc-usage-report>"
            ),
        )

    def test_generate_sax_parser_same_parents_with_diff_fields(self):
        yaml_data = """
---
VtepTable:
    rpc: get-interface-information
    args:
        interface-name: "vtep"
        extensive: True
    item: physical-interface
    key: name
    view: VtepView

VtepView:
    fields:
        admin-status: admin-status
        oper-status: oper-status
        link-level-type: link-level-type
        input-bytes: traffic-statistics/input-bytes
        output-bytes: traffic-statistics/output-bytes
        input-errors: input-error-list/input-errors
        output-errors: output-error-list/output-errors
        carrier-transitions: output-error-list/carrier-transitions
        vtep-tunnel-stats: VtepTunnelTable

VtepTunnelTable:
    item: logical-interface
    key: name
    view: VtepTunnelView

VtepTunnelView:
    fields:
        vtep-type: vtep-info/vtep-type
        vtep-address: vtep-info/vtep-address
        tunnel-input-bytes: traffic-statistics/input-bytes
        tunnel-output-bytes: traffic-statistics/output-bytes
    """
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = VtepTable(self.dev)
        data = generate_sax_parser_input(tbl)
        self.assertEqual(data.tag, "physical-interface")
        self.assertEqual(
            len(etree.tostring(data)),
            len(
                b"<physical-interface><name/><admin-status/><oper-status/>"
                b"<link-level-type/><traffic-statistics><input-bytes/>"
                b"<output-bytes/></traffic-statistics><input-error-list>"
                b"<input-errors/></input-error-list><output-error-list>"
                b"<output-errors/><carrier-transitions/></output-error-list>"
                b"<logical-interface><name/><vtep-info><vtep-type/><vtep-address/>"
                b"</vtep-info><traffic-statistics><input-bytes/><output-bytes/>"
                b"</traffic-statistics></logical-interface></physical-interface>"
            ),
        )

    @patch("jnpr.junos.Device.execute")
    def test_key_pipe_delim_with_Null(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        yaml_data = """
---
UTMStatusTable:
    rpc: show-utmd-status
    item: //utmd-status
    view: UTMStatusView
    key: ../re-name | Null

UTMStatusView:
    fields:
        running: { running: flag }
    """
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = UTMStatusTable(self.dev)
        data = tbl.get()
        self.assertEqual(
            json.loads(data.to_json()),
            {"node0": {"running": True}, "node1": {"running": True}},
        )

    @patch("jnpr.junos.Device.execute")
    def test_key_pipe_delim_with_Null_use_Null(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        yaml_data = """
---
UTMStatusTable:
    rpc: show-utmd-status_use_Null
    item: //utmd-status
    view: UTMStatusView
    key: ../re-name | Null

UTMStatusView:
    fields:
        running: { running: flag }
    """
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = UTMStatusTable(self.dev)
        data = tbl.get()
        self.assertEqual(json.loads(data.to_json()), {"running": True})

    @patch("jnpr.junos.Device.execute")
    def test_key_and_item_pipe_delim_with_Null_use_Null(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        yaml_data = """
---
UTMStatusTable:
    rpc: show-utmd-status_use_Null
    item: //multi-routing-engine-item/utmd-status | //utmd-status
    view: UTMStatusView
    key: 
      - ../re-name | Null

UTMStatusView:
    fields:
        running: { running: flag }
    """
        globals().update(FactoryLoader().load(yaml.load(yaml_data, Loader=yaml.Loader)))
        tbl = UTMStatusTable(self.dev)
        data = tbl.get()
        self.assertEqual(json.loads(data.to_json()), {"running": True})

    def _read_file(self, fname):
        from ncclient.xml_ import NCElement

        fpath = os.path.join(os.path.dirname(__file__), "rpc-reply", fname)
        foo = open(fpath).read()
        reply = RPCReply(foo)
        reply.parse()
        rpc_reply = NCElement(
            reply, self.dev._conn._device_handler.transform_reply()
        )._NCElement__doc[0]
        return rpc_reply

    def _mock_manager(self, *args, **kwargs):
        if kwargs:
            if args and ("normalize" in kwargs or "filter_xml" in kwargs):
                return self._read_file(args[0].tag + ".xml")
            device_params = kwargs["device_params"]
            device_handler = make_device_handler(device_params)
            session = SSHSession(device_handler)
            return Manager(session, device_handler)

        if args:
            return self._read_file(args[0].tag + ".xml")



__author__ = "Nitin Kumar, Rick Sherman"
__credits__ = "Jeremy Schulman"

import unittest
import nose2
from unittest.mock import patch, MagicMock
import os
import sys

from jnpr.junos import Device
from jnpr.junos.ofacts.routing_engines import facts_routing_engines as routing_engines

from ncclient.manager import Manager, make_device_handler
from ncclient.transport import SSHSession


class TestRoutingEngines(unittest.TestCase):
    @patch("ncclient.manager.connect")
    @patch("jnpr.junos.device.warnings")
    def setUp(self, mock_warnings, mock_connect):
        mock_connect.side_effect = self._mock_manager
        self.dev = Device(
            host="1.1.1.1",
            user="rick",
            password="password123",
            gather_facts=False,
            fact_style="old",
        )
        self.dev.open()
        self.facts = {}
        self.mode = ""
        self.vc = False
        self.vct = False
        self.vcf = False

    @patch("jnpr.junos.Device.execute")
    @unittest.skipIf(sys.platform == "win32", "will work for windows in coming days")
    def test_multi_re_vc(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.mode = "multi"
        self.vc = True
        routing_engines(self.dev, self.facts)
        self.assertTrue(self.facts["vc_capable"])
        self.assertTrue(self.facts["2RE"])
        self.assertEqual(self.facts["RE0-RE1"]["mastership_state"], "backup")

    # This test is for an issue where a MX may return nothing for the vc rpc
    @patch("jnpr.junos.Device.execute")
    def test_vc_info_true(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.mode = "multi"
        self.vc = True
        self.vct = True
        routing_engines(self.dev, self.facts)
        self.assertFalse(self.facts["vc_capable"])
        self.assertTrue(self.facts["2RE"])
        self.assertEqual(self.facts["RE1"]["mastership_state"], "backup")

    @patch("jnpr.junos.Device.execute")
    @unittest.skipIf(sys.platform == "win32", "will work for windows in coming days")
    def test_mixed_mode_vcf(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.mode = "multi"
        self.vc = True
        self.vcf = True
        routing_engines(self.dev, self.facts)
        self.assertTrue(self.facts["vc_fabric"])
        self.assertEqual(self.facts["vc_mode"], "Mixed")

    @patch("jnpr.junos.Device.execute")
    def test_multi_instance(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.mode = "multi"
        routing_engines(self.dev, self.facts)
        self.assertTrue(self.facts["2RE"])

    @patch("jnpr.junos.Device.execute")
    def test_master(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.mode = "master"
        routing_engines(self.dev, self.facts)
        self.assertEqual(self.facts["RE0"]["mastership_state"], "master")

    @patch("jnpr.junos.Device.execute")
    def test_routing_engine_exception_ret_none(self, mock_execute):
        mock_execute.side_effect = self._mock_manager
        self.dev.rpc.get_route_engine_information = MagicMock(side_effect=ValueError)
        self.assertEqual(routing_engines(self.dev, self.facts), None)

    def _read_file(self, fname):
        from ncclient.xml_ import NCElement

        fpath = os.path.join(os.path.dirname(__file__), "rpc-reply", fname)
        foo = open(fpath).read()

        rpc_reply = NCElement(
            foo, self.dev._conn._device_handler.transform_reply()
        )._NCElement__doc[0]
        return rpc_reply

    def _mock_manager(self, *args, **kwargs):
        if kwargs:
            device_params = kwargs["device_params"]
            device_handler = make_device_handler(device_params)
            session = SSHSession(device_handler)
            return Manager(session, device_handler)

        if args:
            if self.vc is True and args[0].tag == "get-virtual-chassis-information":
                if self.vct is True:
                    return True
                elif self.vcf is True:
                    return self._read_file("get-virtual-chassis-information_mmvcf.xml")
                else:
                    return self._read_file("get-virtual-chassis-information.xml")
            return self._read_file(args[0].tag + "_" + self.mode + ".xml")



try:
    import unittest2 as unittest
except ImportError:
    import unittest
import nose2

from lxml.etree import XML

from jnpr.junos.device import Device
from jnpr.junos.utils.config import Config
from jnpr.junos.exception import RpcError, ConfigLoadError
from jnpr.junos.decorators import timeoutDecorator, normalizeDecorator
from jnpr.junos.decorators import ignoreWarnDecorator

from unittest.mock import patch, MagicMock, PropertyMock, call

from ncclient.operations.rpc import RPCError
from ncclient.manager import Manager, make_device_handler
from ncclient.transport import SSHSession
from ncclient.xml_ import qualify

__author__ = "Rick Sherman"


class Test_Decorators(unittest.TestCase):
    @patch("ncclient.manager.connect")
    def setUp(self, mock_connect):
        mock_connect.side_effect = self._mock_manager_setup
        self.dev = Device(
            host="1.1.1.1", user="rick", password="password123", gather_facts=False
        )
        self.dev.open()

    def test_timeout(self):
        with patch(
            "jnpr.junos.Device.timeout", new_callable=PropertyMock
        ) as mock_timeout:
            mock_timeout.return_value = 30

            def function(x):
                return x

            decorator = timeoutDecorator(function)
            decorator(self.dev, dev_timeout=10)
            calls = [call(), call(10), call(30)]
            mock_timeout.assert_has_calls(calls)

    def test_timeout_except(self):
        with patch(
            "jnpr.junos.Device.timeout", new_callable=PropertyMock
        ) as mock_timeout:
            mock_timeout.return_value = 30

            def function(*args, **kwargs):
                raise Exception()

            decorator = timeoutDecorator(function)
            # test to ensure the exception is raised
            with self.assertRaises(Exception):
                decorator(self.dev, dev_timeout=10)
            calls = [call(), call(10), call(30)]
            # verify timeout was set/reset
            mock_timeout.assert_has_calls(calls)

    # Test default of true and passing true keyword
    def test_normalize_true_true(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            self.dev._normalize = True

            def function(x):
                return x

            decorator = normalizeDecorator(function)
            decorator(self.dev, normalize=True)
            self.assertFalse(mock_transform.called)

    # Test default of true and passing true keyword and a func exception
    def test_normalize_true_true_except(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            self.dev._normalize = True

            def function(*args, **kwargs):
                raise Exception()

            decorator = normalizeDecorator(function)
            with self.assertRaises(Exception):
                decorator(self.dev, normalize=True)
            self.assertFalse(mock_transform.called)

    # Test default of True and passing false keyword
    def test_normalize_true_false(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            mock_transform.return_value = "o.g."
            self.dev._normalize = True

            def function(x):
                return x

            decorator = normalizeDecorator(function)
            decorator(self.dev, normalize=False)
            calls = [call(), call(self.dev._nc_transform), call("o.g.")]
            mock_transform.assert_has_calls(calls)

    # Test default of True and passing false keyword and a func exception
    def test_normalize_true_false_except(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            mock_transform.return_value = "o.g."
            self.dev._normalize = True

            def function(*args, **kwargs):
                raise Exception()

            decorator = normalizeDecorator(function)
            with self.assertRaises(Exception):
                decorator(self.dev, normalize=False)
            calls = [call(), call(self.dev._nc_transform), call("o.g.")]
            mock_transform.assert_has_calls(calls)

    # Test default of false and passing true keyword
    def test_normalize_false_true(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            mock_transform.return_value = "o.g."
            self.dev._normalize = False

            def function(x):
                return x

            decorator = normalizeDecorator(function)
            decorator(self.dev, normalize=True)
            calls = [call(), call(self.dev._norm_transform), call("o.g.")]
            # print mock_transform.call_args_list
            mock_transform.assert_has_calls(calls)

    # Test default of false and passing true keyword and a func exception
    def test_normalize_false_true_except(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            mock_transform.return_value = "o.g."
            self.dev._normalize = False

            def function(*args, **kwargs):
                raise Exception()

            decorator = normalizeDecorator(function)
            with self.assertRaises(Exception):
                decorator(self.dev, normalize=True)
            calls = [call(), call(self.dev._norm_transform), call("o.g.")]
            # print mock_transform.call_args_list
            mock_transform.assert_has_calls(calls)

    # Test default of false and passing false keyword
    def test_normalize_false_false(self):
        with patch(
            "jnpr.junos.Device.transform", new_callable=PropertyMock
        ) as mock_transform:
            self.dev._normalize = False

            def function(x):
                return x

            decorator = normalizeDecorator(function)
            decorator(self.dev, normalize=False)
            self.assertFalse(mock_transform.called)

    # Test default with ignore_warning not present.
    def test_ignore_warning_missing(self):
        def method(self, x):
            return x

        decorator = ignoreWarnDecorator(method)
        response = decorator(self.dev, "foo")
        self.assertEqual("foo", response)

    # Test default with ignore_warning=False.
    def test_ignore_warning_false(self):
        def method(self, x):
            return x

        decorator = ignoreWarnDecorator(method)
        response = decorator(self.dev, "foo", ignore_warning=False)
        self.assertEqual("foo", response)

    # Test with ignore_warning=True and only warnings.
    def test_ignore_warning_true_3snf_warnings(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3snf_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protocols ospf
            delete policy-options prefix-list foo
        """
        self.assertTrue(cu.load(config, ignore_warning=True))

    # Test with ignore_warning='statement not found' and 3 snf warnings.
    def test_ignore_warning_string_3snf_warnings(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3snf_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protocols ospf
            delete policy-options prefix-list foo
        """
        self.assertTrue(cu.load(config, ignore_warning="statement not found"))

    # Test with ignore_warning='statement not found', 1 snf warning,
    # and 1 error.
    def test_ignore_warning_string_1snf_warning_1err(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_1snf_warning_1err)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protcols ospf
            delete policy-options prefix-list foo
        """
        with self.assertRaises(ConfigLoadError):
            cu.load(config, ignore_warning="statement not found")

    # Test with ignore_warning=True, RpcError with no errs attribute.
    # I haven't seen this from an actual device, so this is a very contrived
    # test.
    def test_ignore_warning_string_1snf_warning_1err(self):
        def method(self, x):
            rpc_error = RPCError(XML("<foo/>"), errs=None)
            raise rpc_error

        decorator = ignoreWarnDecorator(method)
        with self.assertRaises(RPCError):
            decorator(self.dev, "foo", ignore_warning=True)

    # Test with ignore_warning=['foo', 'statement not found'] and
    # three statement not found warnings.
    def test_ignore_warning_list_3snf_warnings(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3snf_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protocols ospf
            delete policy-options prefix-list foo
        """
        self.assertTrue(cu.load(config, ignore_warning=["foo", "statement not found"]))

    # Test with ignore_warning='foo', and three statement not found warnings.
    def test_ignore_warning_string_3snf_no_match(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3snf_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protcols ospf
            delete policy-options prefix-list foo
        """
        with self.assertRaises(ConfigLoadError):
            cu.load(config, ignore_warning="foo")

    # Test with ignore_warning=['foo', 'bar], and
    # three statement not found warnings.
    def test_ignore_warning_list_3snf_no_match(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3snf_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protcols ospf
            delete policy-options prefix-list foo
        """
        with self.assertRaises(ConfigLoadError):
            cu.load(config, ignore_warning=["foo", "bar"])

    # Test with ignore_warning=['foo', 'bar], and
    # three warnings which are 'foo boom', 'boom bar', and 'foo bar'
    def test_ignore_warning_list_3warn_match(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3foobar_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protcols ospf
            delete policy-options prefix-list foo
        """
        self.assertTrue(cu.load(config, ignore_warning=["foo", "bar"]))

    # Test with ignore_warning=['foo', 'foo bar], and
    # three warnings which are 'foo boom', 'boom bar', and 'foo bar'
    def test_ignore_warning_list_3warn_no_match(self):
        self.dev._conn.rpc = MagicMock(side_effect=self._mock_manager_3foobar_warnings)
        cu = Config(self.dev)
        config = """
            delete interfaces ge-0/0/0
            delete protcols ospf
            delete policy-options prefix-list foo
        """
        with self.assertRaises(ConfigLoadError):
            cu.load(config, ignore_warning=["foo", "foo bar"])

    def _mock_manager_setup(self, *args, **kwargs):
        if kwargs:
            device_params = kwargs["device_params"]
            device_handler = make_device_handler(device_params)
            session = SSHSession(device_handler)
            return Manager(session, device_handler)

    def _mock_manager_3snf_warnings(self, *args, **kwargs):
        cmd = """
        <load-configuration action="set" format="text">
            <configuration-set>
                delete interfaces ge-0/0/0
                delete protocols ospf
                delete policy-options prefix-list foo
            </configuration-set>
        </load-configuration>
        """
        rsp_string = """
<rpc-reply xmlns="urn:ietf:params:xml:ns:netconf:base:1.0" xmlns:junos="http://xml.juniper.net/junos/16.1R4/junos" xmlns:nc="urn:ietf:params:xml:ns:netconf:base:1.0" message-id="urn:uuid:1f3dfa00-3434-414a-8aa8-0073590c5812">
<load-configuration-results>
<rpc-error>
<error-severity>warning</error-severity>
<error-message>
statement not found
</error-message>
</rpc-error>
<rpc-error>
<error-severity>warning</error-severity>
<error-message>
statement not found
</error-message>
</rpc-error>
<rpc-error>
<error-severity>warning</error-severity>
<error-message>
statement not found
</error-message>
</rpc-error>
<ok/>
</load-configuration-results>
</rpc-reply>
        """
        rsp = XML(rsp_string)
        errors = []
        for err in rsp.findall(".//" + qualify("rpc-error")):
            errors.append(RPCError(err))
        raise RPCError(rsp, errs=errors)

    def _mock_manager_3foobar_warnings(self, *args, **kwargs):
        cmd = """
        <load-configuration action="set" format="text">
            <configuration-set>
                delete interfaces ge-0/0/0
                delete protocols ospf
                delete policy-options prefix-list foo
            </configuration-set>
        </load-configuration>
        """
        rsp_string = """
        <rpc-reply xmlns="urn:ietf:params:xml:ns:netconf:base:1.0" xmlns:junos="http://xml.juniper.net/junos/16.1R4/junos" xmlns:nc="urn:ietf:params:xml:ns:netconf:base:1.0" message-id="urn:uuid:1f3dfa00-3434-414a-8aa8-0073590c5812">
            <load-configuration-results>
                <rpc-error>
                    <error-severity>warning</error-severity>
                    <error-message>
                        foo boom
                    </error-message>
                </rpc-error>
                <rpc-error>
                    <error-severity>warning</error-severity>
                    <error-message>
                        boom bar
                    </error-message>
                </rpc-error>
                <rpc-error>
                    <error-severity>warning</error-severity>
                    <error-message>
                        foo bar
                    </error-message>
                </rpc-error>
                <ok/>
            </load-configuration-results>
        </rpc-reply>
        """
        rsp = XML(rsp_string)
        errors = []
        for err in rsp.findall(".//" + qualify("rpc-error")):
            errors.append(RPCError(err))
        raise RPCError(rsp, errs=errors)

    def _mock_manager_1snf_warning_1err(self, *args, **kwargs):
        cmd = """
        <load-configuration action="set" format="text">
            <configuration-set>
                delete interfaces ge-0/0/0
                delete protcols ospf
                delete policy-options prefix-list foo
            </configuration-set>
        </load-configuration>
        """
        rsp_string = """
        <rpc-reply xmlns="urn:ietf:params:xml:ns:netconf:base:1.0" xmlns:junos="http://xml.juniper.net/junos/16.1R4/junos" xmlns:nc="urn:ietf:params:xml:ns:netconf:base:1.0" message-id="urn:uuid:1f3dfa00-3434-414a-8aa8-0073590c5812">
            <load-configuration-results>
                <rpc-error>
                    <error-severity>warning</error-severity>
                    <error-message>
                    statement not found
                    </error-message>
                </rpc-error>
                <rpc-error>
                    <error-type>protocol</error-type>
                    <error-tag>operation-failed</error-tag>
                    <error-severity>error</error-severity>
                    <error-message>syntax error</error-message>
                    <error-info>
                    <bad-element>protcols</bad-element>
                    </error-info>
                </rpc-error>
                <ok/>
            </load-configuration-results>
        </rpc-reply>
        """
        rsp = XML(rsp_string)
        errors = []
        for err in rsp.findall(".//" + qualify("rpc-error")):
            errors.append(RPCError(err))
        raise RPCError(rsp, errs=errors)



# Tests in this file corresponds to /jnpr/junos/__init__.py

import unittest
import sys

import nose2
from unittest.mock import patch

__author__ = "Nitin Kumar"
__credits__ = "Jeremy Schulman"


class TestJunosInit(unittest.TestCase):
    def test_warning(self):
        print(sys.modules["sys"].version_info)
        with patch.object(sys.modules["sys"], "version_info", (3, 8, 0)) as mock_sys:
            from jnpr import junos

            self.assertEqual(mock_sys, (3, 8, 0))



__author__ = "mwiget"

import unittest

import socket
from jnpr.junos import Device


class TestDeviceSsh(unittest.TestCase):
    def tearDown(self):
        self.dev.close()

    def test_device_open_key_pass(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        port = 2200
        s.bind(("", port))
        s.listen(5)
        print("\nListening on port %d for incoming sessions ..." % (port))
        sock_fd = 0
        while True:
            client, addr = s.accept()
            print("Got a connection from %s:%d!" % (addr[0], addr[1]))
            sock_fd = self.__launch_junos_proxy(client, addr)

            print("Logging in ...")

            self.dev = Device(
                host=None,
                sock_fd=sock_fd,
                user="jenkins",
                ssh_private_key_file="/var/lib/jenkins/.ssh/passkey",
                passwd="password123",
            )

            print("requesting info...")
            self.dev.open()
            self.assertEqual(self.dev.connected, True)
            print("Hostname is %s" % (self.dev.facts["hostname"]))
            assert self.dev.facts["hostname"] == "highlife"
            break

    def __launch_junos_proxy(self, client, addr):
        val = {"MSG-ID": None, "MSG-VER": None, "DEVICE-ID": None}
        msg = ""
        count = 3
        while len(msg) < 100 and count > 0:
            c = client.recv(1)
            if c == "\r":
                continue

            if c == "\n":
                count -= 1
                if msg.find(":"):
                    (key, value) = msg.split(": ")
                    val[key] = value
                    msg = ""
            else:
                msg += c

        print("MSG %s %s %s" % (val["MSG-ID"], val["MSG-VER"], val["DEVICE-ID"]))
        return client.fileno()



__author__ = "Nitin Kumar, Rick Sherman"
__credits__ = "Jeremy Schulman"

import unittest
import nose2
from unittest.mock import patch, MagicMock
from lxml import etree

from jnpr.junos.ofacts.domain import facts_domain
from jnpr.junos import Device
from jnpr.junos.exception import RpcError


class TestDomain(unittest.TestCase):
    @patch("ncclient.manager.connect")
    @patch("jnpr.junos.device.warnings")
    def setUp(self, mock_warnings, mock_connect):
        self.dev = Device(
            host="1.1.1.1",
            user="rick",
            password="password123",
            gather_facts=False,
            fact_style="old",
        )
        self.dev.open()
        self.facts = {}

    @patch("jnpr.junos.ofacts.domain.FS.cat")
    def test_resolv_conf(self, mock_fs_cat):
        mock_fs_cat.return_value = """# domain juniper.net
        search englab.juniper.net spglab.juniper.net juniper.net jnpr.net
        nameserver 10.11.12.13
        """
        self.facts["hostname"] = "test"
        facts_domain(self.dev, self.facts)
        self.assertEqual(self.facts["domain"], "juniper.net")
        self.assertEqual(self.facts["fqdn"], "test.juniper.net")

    @patch("jnpr.junos.ofacts.domain.FS.cat")
    def test_resolv_conf_no_domain(self, mock_fs_cat):
        mock_fs_cat.return_value = """
        search englab.juniper.net spglab.juniper.net juniper.net jnpr.net
        nameserver 10.11.12.13
        """
        self.facts["hostname"] = "test"
        facts_domain(self.dev, self.facts)
        self.assertEqual(self.facts["domain"], None)
        self.assertEqual(self.facts["fqdn"], "test")

    @patch("jnpr.junos.ofacts.domain.FS.cat")
    def test_resolv_conf_file_absent_under_etc(self, mock_fs_cat):
        mock_fs_cat.side_effect = [None, "domain juniper.net"]
        self.facts["hostname"] = "test"
        facts_domain(self.dev, self.facts)
        self.assertEqual(self.facts["domain"], "juniper.net")
        self.assertEqual(self.facts["fqdn"], "test.juniper.net")

    def test_domain_in_configuration(self):
        xmldata = etree.XML(
            """<configuration><system>
                <domain-name>testing.net</domain-name>
                </system></configuration>"""
        )
        self.dev.rpc.get_config = MagicMock(side_effect=xmldata)
        self.facts["hostname"] = "test"
        facts_domain(self.dev, self.facts)
        self.assertEqual(self.facts["domain"], "testing.net")
        self.assertEqual(self.facts["fqdn"], "test.testing.net")

    @patch("jnpr.junos.ofacts.domain.FS.cat")
    def test_domain_rpc_error(self, mock_fs_cat):
        self.dev.rpc.get_config = MagicMock(side_effect=RpcError)
        mock_fs_cat.return_value = """# domain juniper.net
        search englab.juniper.net spglab.juniper.net juniper.net jnpr.net
        nameserver 10.11.12.13
        """
        self.facts["hostname"] = "test"
        facts_domain(self.dev, self.facts)
        self.assertEqual(self.facts["domain"], "juniper.net")
        self.assertEqual(self.facts["fqdn"], "test.juniper.net")



__author__ = "rsherman, vnitinv"

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from jnpr.junos.op.routes import RouteTable
import json


class TestTable(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        from jnpr.junos import Device

        self.dev = Device(host="xxxx", user="jenkins", password="password")
        self.dev.open()

    @classmethod
    def tearDownClass(self):
        self.dev.close()

    def test_table_union(self):
        tbl = RouteTable(self.dev)
        tbl.get()
        self.assertEqual(tbl[0].via, "em0.0")

    def test_table_json(self):
        tbl = RouteTable(self.dev)
        tbl.get("10.48.21.71")
        self.assertEqual(
            json.loads(tbl.to_json())["10.48.21.71/32"]["protocol"], "Local"
        )



__author__ = "Nitin Kumar, Rick Sherman"
__credits__ = "Jeremy Schulman"

import unittest
import nose2

from jnpr.junos import Device
from jnpr.junos.utils.util import Util

from unittest.mock import patch


class TestUtil(unittest.TestCase):
    @patch("ncclient.manager.connect")
    def setUp(self, mock_connect):
        self.dev = Device(
            host="1.1.1.1", user="nitin", password="password123", gather_facts=False
        )
        self.dev.open()
        self.util = Util(self.dev)

    def test_repr(self):
        self.assertEqual(repr(self.util), "jnpr.junos.utils.Util(1.1.1.1)")

    def test_dev_setter_exception(self):
        def mod_dev():
            self.util.dev = "abc"

        self.assertRaises(RuntimeError, mod_dev)

    def test_rpc_setter_exception(self):
        def mod_rpc():
            self.util.rpc = "abc"

        self.assertRaises(RuntimeError, mod_rpc)
